package org.nuiton.topia.test.ano1882;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import org.apache.commons.lang3.builder.ToStringBuilder;
import org.nuiton.i18n.I18n;
import org.nuiton.topia.TopiaException;
import org.nuiton.topia.persistence.EntityVisitor;
import org.nuiton.topia.persistence.TopiaEntity;
import org.nuiton.topia.persistence.TopiaEntityAbstract;
import org.nuiton.topia.persistence.util.TopiaEntityHelper;

public abstract class FrenchCompanyAbstract extends TopiaEntityAbstract implements FrenchCompany {

    /**
     * Nom de l'attribut en BD : sIREN
     */
    protected SIREN sIREN;

    /**
     * Nom de l'attribut en BD : sIRET
     */
    protected Collection<SIRET> sIRET;

    /**
     * Nom de l'attribut en BD : sIREN2
     */
    protected SIREN SIREN2;

    /**
     * Nom de l'attribut en BD : sIRET2
     */
    protected Collection<SIRET> SIRET2;

    private static final long serialVersionUID = 3846746116233181537L;

    static {
    
        I18n.n_("topia.test.common.frenchCompany");
        I18n.n_("topia.test.common.sIREN");
        I18n.n_("topia.test.common.sIRET");
        I18n.n_("topia.test.common.SIREN2");
        I18n.n_("topia.test.common.SIRET2");
    }

    @Override
    public void accept(EntityVisitor visitor) throws TopiaException {
        visitor.start(this);
        visitor.visit(this, PROPERTY_S_IREN, SIREN.class, sIREN);
        visitor.visit(this, PROPERTY_S_IRET, Collection.class, SIRET.class, sIRET);
        visitor.visit(this, PROPERTY_SIREN2, SIREN.class, SIREN2);
        visitor.visit(this, PROPERTY_SIRET2, Collection.class, SIRET.class, SIRET2);
        visitor.end(this);
    }

    @Override
    public void setsIREN(SIREN sIREN) {
        SIREN oldValue = this.sIREN;
        fireOnPreWrite(PROPERTY_S_IREN, oldValue, sIREN);
        this.sIREN = sIREN;
        fireOnPostWrite(PROPERTY_S_IREN, oldValue, sIREN);
    }

    @Override
    public SIREN getsIREN() {
        fireOnPreRead(PROPERTY_S_IREN, sIREN);
        SIREN result = this.sIREN;
        fireOnPostRead(PROPERTY_S_IREN, sIREN);
        return result;
    }

    @Override
    public void addsIRET(SIRET sIRET) {
        fireOnPreWrite(PROPERTY_S_IRET, null, sIRET);
        if (this.sIRET == null) {
            this.sIRET = new ArrayList<SIRET>();
        }

        this.sIRET.add(sIRET);
        fireOnPostWrite(PROPERTY_S_IRET, this.sIRET.size(), null, sIRET);
    }

    @Override
    public void addAllsIRET(Collection<SIRET> sIRET) {
        if (sIRET == null) {
            return;
        }
        for (SIRET item : sIRET) {
            addsIRET(item);
        }
    }

    @Override
    public void setsIRET(Collection<SIRET> sIRET) {
        // Copy elements to keep data for fire with new reference
        Collection<SIRET> oldValue = this.sIRET != null ? new ArrayList<SIRET>(this.sIRET) : null;
        fireOnPreWrite(PROPERTY_S_IRET, oldValue, sIRET);
        this.sIRET = sIRET;
        fireOnPostWrite(PROPERTY_S_IRET, oldValue, sIRET);
    }

    @Override
    public void removesIRET(SIRET sIRET) {
        fireOnPreWrite(PROPERTY_S_IRET, sIRET, null);
        if (this.sIRET == null || !this.sIRET.remove(sIRET)) {
            throw new IllegalArgumentException("List does not contain given element");
        }

        fireOnPostWrite(PROPERTY_S_IRET, this.sIRET.size() + 1, sIRET, null);
    }

    @Override
    public void clearsIRET() {
        if (this.sIRET == null) {
            return;
        }
        Collection<SIRET> oldValue = new ArrayList<SIRET>(this.sIRET);
        fireOnPreWrite(PROPERTY_S_IRET, oldValue, this.sIRET);
        this.sIRET.clear();
        fireOnPostWrite(PROPERTY_S_IRET, oldValue, this.sIRET);
    }

    @Override
    public Collection<SIRET> getsIRET() {
        return sIRET;
    }

    @Override
    public SIRET getsIRETByTopiaId(String topiaId) {
        return TopiaEntityHelper.getEntityByTopiaId(sIRET, topiaId);
    }

    @Override
    public int sizesIRET() {
        if (sIRET == null) {
            return 0;
        }
        return sIRET.size();
    }

    @Override
    public boolean issIRETEmpty() {
        int size = sizesIRET();
        return size == 0;
    }

    @Override
    public void setSIREN2(SIREN SIREN2) {
        SIREN oldValue = this.SIREN2;
        fireOnPreWrite(PROPERTY_SIREN2, oldValue, SIREN2);
        this.SIREN2 = SIREN2;
        fireOnPostWrite(PROPERTY_SIREN2, oldValue, SIREN2);
    }

    @Override
    public SIREN getSIREN2() {
        fireOnPreRead(PROPERTY_SIREN2, SIREN2);
        SIREN result = this.SIREN2;
        fireOnPostRead(PROPERTY_SIREN2, SIREN2);
        return result;
    }

    @Override
    public void addSIRET2(SIRET SIRET2) {
        fireOnPreWrite(PROPERTY_SIRET2, null, SIRET2);
        if (this.SIRET2 == null) {
            this.SIRET2 = new ArrayList<SIRET>();
        }

        this.SIRET2.add(SIRET2);
        fireOnPostWrite(PROPERTY_SIRET2, this.SIRET2.size(), null, SIRET2);
    }

    @Override
    public void addAllSIRET2(Collection<SIRET> SIRET2) {
        if (SIRET2 == null) {
            return;
        }
        for (SIRET item : SIRET2) {
            addSIRET2(item);
        }
    }

    @Override
    public void setSIRET2(Collection<SIRET> SIRET2) {
        // Copy elements to keep data for fire with new reference
        Collection<SIRET> oldValue = this.SIRET2 != null ? new ArrayList<SIRET>(this.SIRET2) : null;
        fireOnPreWrite(PROPERTY_SIRET2, oldValue, SIRET2);
        this.SIRET2 = SIRET2;
        fireOnPostWrite(PROPERTY_SIRET2, oldValue, SIRET2);
    }

    @Override
    public void removeSIRET2(SIRET SIRET2) {
        fireOnPreWrite(PROPERTY_SIRET2, SIRET2, null);
        if (this.SIRET2 == null || !this.SIRET2.remove(SIRET2)) {
            throw new IllegalArgumentException("List does not contain given element");
        }

        fireOnPostWrite(PROPERTY_SIRET2, this.SIRET2.size() + 1, SIRET2, null);
    }

    @Override
    public void clearSIRET2() {
        if (this.SIRET2 == null) {
            return;
        }
        Collection<SIRET> oldValue = new ArrayList<SIRET>(this.SIRET2);
        fireOnPreWrite(PROPERTY_SIRET2, oldValue, this.SIRET2);
        this.SIRET2.clear();
        fireOnPostWrite(PROPERTY_SIRET2, oldValue, this.SIRET2);
    }

    @Override
    public Collection<SIRET> getSIRET2() {
        return SIRET2;
    }

    @Override
    public SIRET getSIRET2ByTopiaId(String topiaId) {
        return TopiaEntityHelper.getEntityByTopiaId(SIRET2, topiaId);
    }

    @Override
    public int sizeSIRET2() {
        if (SIRET2 == null) {
            return 0;
        }
        return SIRET2.size();
    }

    @Override
    public boolean isSIRET2Empty() {
        int size = sizeSIRET2();
        return size == 0;
    }

    @Override
    public List<TopiaEntity> getAggregate() throws TopiaException {
        List<TopiaEntity> tmp = new ArrayList<TopiaEntity>();

        // pour tous les attributs rechecher les composites et les class d'asso
        // on les ajoute dans tmp

        // on refait un tour sur chaque entity de tmp pour recuperer leur
        // composite
        List<TopiaEntity> result = new ArrayList<TopiaEntity>();
        for (TopiaEntity entity : tmp) {
            result.add(entity);
            result.addAll(entity.getAggregate());
        }
        return result;
    }

    @Override
    public List<TopiaEntity> getComposite() throws TopiaException {
        List<TopiaEntity> tmp = new ArrayList<TopiaEntity>();

        // pour tous les attributs rechecher les composites et les class d'asso
        // on les ajoute dans tmp
        if (getsIRET() != null) {
              tmp.addAll(getsIRET());
           }

        // on refait un tour sur chaque entity de tmp pour recuperer leur
        // composite
        List<TopiaEntity> result = new ArrayList<TopiaEntity>();
        for (TopiaEntity entity : tmp) {
            if (entity != null) {
                result.add(entity);
                result.addAll(entity.getComposite());
            }
        }
        return result;
    }

    @Override
    public String toString() {
        String result = new ToStringBuilder(this).
            append(PROPERTY_S_IREN, this.sIREN).
            append(PROPERTY_S_IRET, this.sIRET).
            append(PROPERTY_SIREN2, this.SIREN2).
            append(PROPERTY_SIRET2, this.SIRET2).
         toString();
        return result;
    }

} //FrenchCompanyAbstract
